/*
 * 判断obj是否为一个整数
 */
function isInteger(obj) {
    return Math.floor(obj) === obj
}

/*
 * 将一个浮点数转成整数，返回整数和倍数。如 3.14 >> 314，倍数是 100
 * @param floatNum {number} 小数
 * @return {object}
 *   {times:100, num: 314}
 */
function toInteger(floatNum) {
    if(floatNum === undefined) floatNum = 0
    var ret = {times: 1, num: 0}
    if (isInteger(floatNum)) {
        ret.num = floatNum
        return ret
    }
    var strfi  = floatNum + ''
    var dotPos = strfi.indexOf('.')
    var len    = strfi.substr(dotPos+1).length
    var times  = Math.pow(10, len)
    var intNum;
    if(floatNum < 0){
        intNum = parseInt(floatNum * times)
    }else{
        intNum = parseInt(floatNum * times + 0.5, 10)
    }
    // var intNum = parseInt(floatNum * times + 0.5, 10)//屏蔽掉（因为会将类似-1.53188转换成-1531870）
    // var intNum = parseInt(floatNum * times)
    ret.times  = times
    ret.num    = intNum
    return ret
}

/**
 * 
 * @param num {Number} 数字参数      
 * @param digits {Number} 保留的有效位数
 * 返回字符串的
 */
/* function goFixed(num,digits){
    var sNum = Math.round(num*Math.pow(10, digits)).toString();
    var sLen = sNum.length;
    if (sLen <= digits) {
        var fNum = digits - sLen + 1;
        for (let i=0; i< fNum; i++) {
            sNum = '0' + sNum;
        }
    }
    return `${sNum.slice(0,sNum.length-digits)}.${sNum.slice(-digits)}`;
} */
// 返回保留有效数字精度的 数字
function goFixed(num , digits) {
    var pow = Math.pow(10, digits);
    return Math.round(num*pow)/pow;
}

/*
 * 核心方法，实现加减乘除运算，确保不丢失精度
 * 思路：把小数放大为整数（乘），进行算术运算，再缩小为小数（除）
 *
 * @param a {number} 运算数1
 * @param b {number} 运算数2
 * @param digits {number} 精度，保留的小数点数，比如 2, 即保留为两位小数
 * @param op {string} 运算类型，有加减乘除（add/subtract/multiply/divide）
 *
 */
function operation(a, b, digits, op) {
    var o1 = toInteger(a)
    var o2 = toInteger(b)
    var n1 = o1.num
    var n2 = o2.num
    var t1 = o1.times
    var t2 = o2.times
    var max = t1 > t2 ? t1 : t2
    var result = null
    switch (op) {
        case 'add':
            if (t1 === t2) { // 两个小数位数相同
                result = n1 + n2
            } else if (t1 > t2) { // o1 小数位 大于 o2
                result = n1 + n2 * (t1 / t2)
            } else { // o1 小数位 小于 o2
                result = n1 * (t2 / t1) + n2
            }
            if(digits >= 0 && digits <= 20){
                return goFixed(result / max, digits);
            }
            return result / max
        case 'subtract':
            if (t1 === t2) {
                result = n1 - n2
            } else if (t1 > t2) {
                result = n1 - n2 * (t1 / t2)
            } else {
                result = n1 * (t2 / t1) - n2
            }
            if(digits >= 0 && digits <= 20){
                return goFixed(result / max, digits);
            }
            return result / max
        case 'multiply':
            result = (n1 * n2) / (t1 * t2)
            if(digits >= 0 && digits <= 20){
                return goFixed(result, digits);
            }
            return result
        case 'divide':
            if(n1 === 0 || n2 === 0) return 0;
            result = (n1 / n2) * (t2 / t1)
            if(digits >= 0 && digits <= 20){
                return goFixed(result, digits);
            }
            return result
    }
}

function dateAdd(interval,number,date,add)  {//如果add为true则加否则减
    switch(interval) {
        case   "年"   :   {
            if(add) date.setFullYear(date.getFullYear()+number);
            else date.setFullYear(date.getFullYear()-number);
            return   date;
            break;
        }
        case   "月"   :   {
            if(add) date.setMonth(date.getMonth()+number);
            else date.setMonth(date.getMonth()-number);
            return   date;
            break;
        }
        case   "日"   :   {
            if(add) date.setDate(date.getDate()+number);
            else date.setDate(date.getDate()-number);
            return   date;
            break;
        }
        default   :   {
            date.setDate(date.getDate()+number);
            return   date;
            break;
        }
    }

}
// 加减乘除的四个接口
function numberAdd(a, b, digits) {
    return operation(a, b, digits, 'add')
}
function numberSub(a, b, digits) {
    return operation(a, b, digits, 'subtract')
}
function numberMul(a, b, digits) {
    return operation(a, b, digits, 'multiply')
}
function numberDiv(a, b, digits) {
    return operation(a, b, digits, 'divide')
}

// exports
export default {
    dateAdd,
    numberAdd,
    numberSub,
    numberMul,
    numberDiv
}